package cerebral.colors;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ScrollPaneConstants;
import javax.swing.ToolTipManager;

import publicdomain.ScrollableFlowPanel;

public class Gallery extends JPanel implements MouseListener {

    private static final long serialVersionUID = 1L;

    private List coreScales = new ArrayList();
    private List coreRecommendations = new ArrayList();

    private List userScales;

    private ScrollableFlowPanel gallery = new ScrollableFlowPanel();

    public Gallery(String s) {
        String[] us = s.split("=");
        userScales = new ArrayList();
        for (int i = 0; i < us.length; i++) {
            userScales.add(us[i]);
        }
        init(userScales);

        ToolTipManager.sharedInstance().setInitialDelay(0);
    }

    public void init(List userScales) {
        this.userScales = userScales;
        setBackground(Color.white);
        setLayout(new GridBagLayout());
        GridBagConstraints c = new GridBagConstraints();
        c.gridx = 0;
        c.gridy = 0;
        c.anchor = GridBagConstraints.LINE_START;
        JLabel instructions = new JLabel(
                "or select a new color scale from the gallery");
        add(instructions, c);

        c.gridx = 0;
        c.gridy = 1;
        c.weighty = 1.0;
        c.weightx = 1.0;
        c.fill = GridBagConstraints.BOTH;
        JScrollPane galleryPane = new JScrollPane();
        galleryPane
                .setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED);
        galleryPane
                .setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
        galleryPane.setPreferredSize(new Dimension(200, 200));
        galleryPane.setViewportView(gallery);
        galleryPane.getViewport().setBackground(Color.white);
        add(galleryPane, c);

        coreScales.add("c,-16.0,-15034815,c,0.0,-2680548,t,16.0,");
        coreRecommendations.add("<html><b>Recommended use</b>"+
        		"<p>Low-confidence data with large error terms"+
        		"<p>"+
        		"<p>Color-blind friendly: No");
        
        coreScales.add("c,-16.0,-103,c,0.0,-13861962,t,16.0,");
        coreRecommendations.add("<html><b>Recommended use</b>"+
        		"<p>Low-confidence data with large error terms"+
        		"<p>"+
        		"<p>Color-blind friendly: Somewhat");
        coreScales.add("c,-16.0,-941248,c,0.0,-6713661,t,16.0,");
        coreRecommendations.add("<html><b>Recommended use</b>"+
        		"<p>Low-confidence data with large error terms"+
        		"<p>"+
        		"<p>Color-blind friendly: Yes");
        
        coreScales.add("l,-2.5,-15034815,-65,l,0.0,-65,-2680548,t,2.5,");
        coreRecommendations.add("<html><b>Recommended use</b>"+
        		"<p>High-confidence data with postive and negative values"+
        		"<p>"+
        		"<p>Color-blind friendly: No");
        
        coreScales.add("l,-2.5,-1679103,-526345,l,0.0,-526345,-10601319,t,2.5,");
        coreRecommendations.add("<html><b>Recommended use</b>"+
        		"<p>High-confidence data with postive and negative values"+
        		"<p>"+
        		"<p>Color-blind friendly: Yes");
        
        coreScales.add("c,-2.5,-15034815,c,-2.0,-5842582,c,-1.5,-65,c,1.5,-151967,c,2.0,-2680548,t,2.5,");
        coreRecommendations.add("<html><b>Recommended use</b>"+
        		"<p>Mid-confidence data with postive and negative values"+
        		"<p>"+
        		"<p>Color-blind friendly: No");
        
        coreScales.add("c,-2.5,-1679103,c,-2.0,-149405,c,-1.5,-526345,c,1.5,-5067822,c,2.0,-10601319,t,2.5,");
        coreRecommendations.add("<html><b>Recommended use</b>"+
        		"<p>Mid-confidence data with postive and negative values"+
        		"<p>"+
        		"<p>Color-blind friendly: Yes");
        
        coreScales.add("l,0.0,-69415,-5046272,t,5.0,");
        coreRecommendations.add("<html><b>Recommended use</b>"+
        		"<p>High-confidence data with all postive values"+
        		"<p>"+
        		"<p>Color-blind friendly: Yes");
        
        coreScales.add("l,0.0,-52,-13524140,t,5.0,");
        coreRecommendations.add("<html><b>Recommended use</b>"+
        		"<p>High-confidence data with all negative values"+
        		"<p>"+
        		"<p>Color-blind friendly: Yes");
        
        coreScales.add("c,0.0,-69415,c,1.0,-144246,c,2.0,-225959,c,3.0,-1881549,c,4.0,-5046272,t,5.0,");
        coreRecommendations.add("<html><b>Recommended use</b>"+
        		"<p>Mid-confidence data with all postive values"+
        		"<p>"+
        		"<p>Color-blind friendly: Yes");
        
        coreScales.add("c,0.0,-1181445,c,1.0,-5053726,c,2.0,-10042716,c,3.0,-13852065,c,4.0,-16749268,t,5.0,");
        coreRecommendations.add("<html><b>Recommended use</b>"+
        		"<p>Mid-confidence data with all negative values"+
        		"<p>"+
        		"<p>Color-blind friendly: Yes");
        
        coreScales.add("c,0.5,-14967177,c,1.5,-2531582,c,2.5,-9080653,c,3.5,-1627766,c,4.5,-10050018,c,5.5,-1660158,c,6.5,-5868003,t,7.5,");
        coreRecommendations.add("<html><b>Recommended use</b>"+
        		"<p>Data values 1,2,3,4,5,6,7 each represent something different"+
        		"<p>"+
        		"<p>Color-blind friendly: No");
        
        coreScales.add("c,0.5,-8402561,c,1.5,-4280620,c,2.5,-147322,c,3.5,-103,c,4.5,-13079376,c,5.5,-1047937,c,6.5,-4236521,t,7.5,");
        coreRecommendations.add("<html><b>Recommended use</b>"+
        		"<p>Data values 1,2,3,4,5,6,7 each represent something different"+
        		"<p>"+
        		"<p>Color-blind friendly: Somewhat");
        
        
        
        /*
        coreScales.add(new String(
                "c,-16.0,-65536,c,-1.5,-52,c,1.5,-16711936,t,16.0,"));
        coreScales
                .add(new String(
                        "l,-16.0,-65536,-14680064,c,-1.5,-52,l,1.5,-16769024,-16711936,t,16.0,"));
        coreScales
                .add(new String(
                        "s,-16.0,-65536,8.0,false,c,-1.5,-52,s,1.5,-16711936,8.0,true,t,16.0,"));
        coreScales
                .add(new String(
                        "c,-3.5,-65536,c,-2.5,-32640,c,-1.5,-52,c,1.5,-8323200,c,2.5,-16711936,t,3.5,"));
        coreScales.add(new String(
                "l,-16.0,-65536,-14680064,l,0.0,-16769024,-16711936,t,16.0,"));
        coreScales.add(new String("l,0.0,-16777216,-65536,t,16.0,"));
        coreScales.add(new String(
                "l,-2.0,-1679360,-10074061,c,-0.5,-52,l,0.5,-11390106,-10616679,t,2.0,"));
        coreScales.add(new String("c,-2.5,-38144,c,-1.5,-8375040,c,-0.5,-52,c,0.5,-11403136,c,1.5,-5963521,t,2.5,"));

*/
        gallery.setBackground(Color.white);
        createGallery();
    }

    private void createGallery() {
        gallery.removeAll();
        for (int i = userScales.size() - 1; i >= 0; i--) {
            addToGallery((String) userScales.get(i), "");
        }
        for (int i = 0; i < coreScales.size(); i++) {        
            addToGallery((String) coreScales.get(i), (String) coreRecommendations.get(i)) ;
        }
        gallery.validate();
    }

    private void addToGallery(String s, String tooltip) {
        if (s.length() > 0) {
            ColorRange head = ColorScaleBar.loadFromString(s);
            ColorScaleBar csb = new ColorScaleBar(head);
            if (tooltip.length() > 0){
            	csb.setToolTipText(tooltip);
            }
            gallery.add(csb);
            csb.addMouseListener(this);
        }
    }

    public void addUserScale(String string) {
        userScales.add(string);
        createGallery();
    }

    public void mouseClicked(MouseEvent e) {
        if (e.getSource() instanceof ColorScaleBar) {
            ColorScaleBar csb = (ColorScaleBar) e.getSource();
            firePropertyChange("scaleChosen", null, csb.saveString());
        }
    }

    public String getUserScales() {
        StringBuffer s = new StringBuffer();
        for (int i = 0; i < userScales.size(); i++) {
            s.append(userScales.get(i) + "=");
        }
        return s.toString();
    }

    public void mouseEntered(MouseEvent e) {
        // TODO Auto-generated method stub

    }

    public void mouseExited(MouseEvent e) {
        // TODO Auto-generated method stub

    }

    public void mousePressed(MouseEvent e) {
        // TODO Auto-generated method stub

    }

    public void mouseReleased(MouseEvent e) {
        // TODO Auto-generated method stub

    }

}
