package cerebral.colors;

import java.awt.Color;
import java.awt.Graphics;

import prefuse.util.ColorLib;

public class SigmoidalRange extends ColorRangeComponent {

    private static final long serialVersionUID = 1L;

    float[] hsb;

    double s;

    int color;

    public boolean left;

    public SigmoidalRange(double min, ColorRange lowerRange, int color,
            double s, boolean left) {
        super(min, lowerRange);
        hsb = Color.RGBtoHSB(ColorLib.red(color), ColorLib.green(color),
                ColorLib.blue(color), null);
        this.s = s;
        this.color = color;
        this.left = left;
    }

    public void setColor(int color) {
        hsb = Color.RGBtoHSB(ColorLib.red(color), ColorLib.green(color),
                ColorLib.blue(color), null);
        this.color = color;
    }

    public int getColor(double val) {
        if (isInRange(val)) {
            double frac;
            if (left) {
                frac = 1.0 - (val - min) / (higherRange.getMin() - min);
            } else {
                frac = (val - min) / (higherRange.getMin() - min);
            }
            float i = (float) ((2 / (1 + Math.exp(-1 * s * frac))) - 1);
            return Color.HSBtoRGB(hsb[0], i, i);
        }  else if (val < min) {
            // We're off the left side of the scale. Clip to min
            double frac = (left) ? 1.0 : 0.0;
            float i = (float) ((2 / (1 + Math.exp(-1 * s * frac))) - 1);
            return Color.HSBtoRGB(hsb[0], i, i);
        } else if (higherRange instanceof TerminateRange) {
            double frac = (left) ? 0.0 : 1.0;
            float i = (float) ((2 / (1 + Math.exp(-1 * s * frac))) - 1);
            return Color.HSBtoRGB(hsb[0], i, i);
        } else  {
            return higherRange.getColor(val);
        }
    }

    public void paint(Graphics g) {
        int w = getWidth();
        for (int i = 0; i < w; i++) {
            double val = min + (i / (float) w) * (higherRange.getMin() - min);
            int color = getColor(val);
            g.setColor(ColorLib.getColor(color));
            g.drawLine(i, 0, i, getHeight());
        }
        super.paint(g);
    }

}
