package cerebral.controls;

import java.awt.event.MouseEvent;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import javax.swing.SwingUtilities;

import cerebral.Cerebral;
import cerebral.EdgeBundler;

import prefuse.Display;
import prefuse.Visualization;
import prefuse.visual.AggregateItem;
import prefuse.visual.VisualItem;
import prefuse.controls.ControlAdapter;


public class RubberBandSelect extends ControlAdapter {
    private int downX1, downY1;

    private VisualItem rubberBand;

    Point2D screenPoint = new Point2D.Float();

    Point2D absPoint = new Point2D.Float();

    Rectangle2D rect = new Rectangle2D.Float();

    private Cerebral cytoPrefuse;

    public RubberBandSelect(Cerebral cytoPrefuse, VisualItem rubberBand) {
        this.rubberBand = rubberBand;
        this.cytoPrefuse = cytoPrefuse;
    }

    public void itemPressed(VisualItem item, MouseEvent e) {
        if (item instanceof AggregateItem) {
            mousePressed(e);
        }
    }

    public void itemDragged(VisualItem item, MouseEvent e) {
        if (item instanceof AggregateItem) {
            mouseDragged(e);
        }
    }

    public void itemReleased(VisualItem item, MouseEvent e) {
        if (item instanceof AggregateItem) {
            mouseReleased(e);
        }
    }

    public void mousePressed(MouseEvent e) {
        if (!SwingUtilities.isLeftMouseButton(e))
            return;

        Display d = (Display) e.getComponent();
        screenPoint.setLocation(e.getX(), e.getY());
        d.getAbsoluteCoordinate(screenPoint, absPoint);
        downX1 = (int) absPoint.getX();
        downY1 = (int) absPoint.getY();
        rubberBand.setVisible(true);
    }

    public void mouseDragged(MouseEvent e) {
        if (!SwingUtilities.isLeftMouseButton(e))
            return;

        Display d = (Display) e.getComponent();
        screenPoint.setLocation(e.getX(), e.getY());
        d.getAbsoluteCoordinate(screenPoint, absPoint);
        int x1 = downX1;
        int y1 = downY1;
        int x2 = (int) absPoint.getX();
        int y2 = (int) absPoint.getY();

        float[] bandRect = (float[]) rubberBand.get(VisualItem.POLYGON);
        bandRect[0] = x1;
        bandRect[1] = y1;
        bandRect[2] = x2;
        bandRect[3] = y1;
        bandRect[4] = x2;
        bandRect[5] = y2;
        bandRect[6] = x1;
        bandRect[7] = y2;

        rubberBand.setValidated(false);

        if (x2 < x1) {
            int temp = x2;
            x2 = x1;
            x1 = temp;
        }
        if (y2 < y1) {
            int temp = y2;
            y2 = y1;
            y1 = temp;
        }
        rect.setRect(x1, y1, x2 - x1, y2 - y1);

        Set selectedItems = new HashSet();
        Visualization vis = d.getVisualization();
        Iterator it = vis.getGroup(Cerebral.NODES).tuples();

        while (it.hasNext()) {
            VisualItem item = (VisualItem) it.next();
            if (item.isVisible() && item.getBounds().intersects(rect)) {
                selectedItems.add(item);
            }
        }
        it = vis.getGroup(Cerebral.EDGES).tuples();
        while (it.hasNext()) {
            VisualItem item = (VisualItem) it.next();
            if (item.isVisible() && item.getBounds().intersects(rect)) {
                // Bounding box intersect. Now test precisely
                if (EdgeBundler.edgeIntersectsRect(item, rect))
                    selectedItems.add(item);
            }
        }

        if (!e.isShiftDown()) {
            if (selectedItems.isEmpty()) {
                cytoPrefuse.selectAll();
            } else {
                cytoPrefuse.setSelectedItems(selectedItems);
            }
        } else {
            cytoPrefuse.selectInCytoscape(selectedItems.toArray(), true);
        }
        d.getVisualization().repaint();
    }

    public void mouseReleased(MouseEvent e) {
        if (!SwingUtilities.isLeftMouseButton(e))
            return;
        rubberBand.setVisible(false);
        Display d = (Display) e.getComponent();
        d.getVisualization().repaint();
    }
}
