/**
 * 
 */
package cerebral.layout;

import java.awt.Color;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics2D;
import java.awt.geom.Rectangle2D;
import java.util.Arrays;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Vector;

import cerebral.Cerebral;

import prefuse.Display;
import prefuse.action.Action;
import prefuse.data.tuple.TupleSet;
import prefuse.render.Renderer;
import prefuse.util.ColorLib;
import prefuse.util.FontLib;
import prefuse.util.PrefuseLib;
import prefuse.util.display.PaintListener;
import prefuse.visual.AggregateItem;
import prefuse.visual.DecoratorItem;
import prefuse.visual.NodeItem;
import prefuse.visual.VisualItem;

public class LabelLayout extends Action implements PaintListener {
	private int noScaleFontSize = 12;
	private DecoratorItem[] orderedLabels = new DecoratorItem[0];
	private int xMax;
	private int yMax;
	private int lastFontSize = 0;
	private boolean[][] freeSpace;
	private Comparator sorter;
	private Map functionGroups = new HashMap();
	private float extraSpace;
	private float extraSpaceX;
	private float extraSpaceY;
	private Cerebral cytoPrefuse;
	private Display d;
	private Rectangle2D r = new Rectangle2D.Double();
	private final int GRIDIFY = 30;
	private float aggregateScale;
	private boolean renderAggregateLabels = false;
	
	public LabelLayout(Comparator sorter,
			Cerebral cytoPrefuse) {
		this.sorter = sorter;
		this.cytoPrefuse = cytoPrefuse;
		this.d = (Display) cytoPrefuse;

		cytoPrefuse.getVisualization().getBounds(Cerebral.NODES, r);
		xMax = (int) (r.getMaxX() / GRIDIFY) + 5;
		yMax = (int) (r.getMaxY() / GRIDIFY) + 5;
		freeSpace = new boolean[xMax][yMax];
		setAggregateLabelScale(1.0f);
		}
	
	public void setLabelsToManage(DecoratorItem[] labels){
		orderedLabels = labels;
		for (int i = 0; i < orderedLabels.length; i++) {
			DecoratorItem item = orderedLabels[i];
			Object o = item.getDecoratedItem();
			if (o instanceof NodeItem) {
				VisualItem node = item.getDecoratedItem();
				if (node.getBoolean(Cerebral.DOWNSTREAM_GENE)) {
					Vector functionNeighbours = (Vector) node
							.get(Cerebral.FUNCTION_NEIGHBOURS);

					if (functionNeighbours != null) {
						functionGroups.put(new Integer(node
								.getInt(Cerebral.BIO_FUNCTION)),
								functionNeighbours);
					}
				}
			}
		}		
	}

	public void setAggregateLabelScale(float scale) {
		aggregateScale = scale;
		if (scale > 0.01) {
			renderAggregateLabels = true;
		} else {
			renderAggregateLabels = false;
		}
		setLabelPosition();
		d.repaint();
	}

	public void setExtraSpace(float extraSpace) {
		this.extraSpace = extraSpace;
		setLabelPosition();
		d.repaint();
	}

	public void setLabelPosition() {

		Font defaultFont;

		cytoPrefuse.getVisualization().getBounds(Cerebral.NODES, r);
		int curXMax = (int) (r.getMaxX() / GRIDIFY) + 5;
		int curYMax = (int) (r.getMaxY() / GRIDIFY) + 5;

		if (xMax != curXMax || yMax != curYMax) {
			xMax = curXMax;
			yMax = curYMax;

			freeSpace = new boolean[xMax][yMax];
		}
		extraSpaceX = extraSpace * xMax;
		extraSpaceY = extraSpace * yMax;
	

		int fontSize = (int) (noScaleFontSize * (1 / d.getScale()));

		if (fontSize < noScaleFontSize) {
			fontSize = noScaleFontSize;
		}

		defaultFont = FontLib.getFont("Tahoma", fontSize);

		Color bgColor = d.getBackground();
		Color transparentBG = ColorLib.getColor(bgColor.getRed(), bgColor
				.getGreen(), bgColor.getBlue(), 150);

		for (int i = 0; i < xMax; i++) {
			Arrays.fill(freeSpace[i], true);
		}

		TupleSet selectedItems = null;
        if (m_vis != null) {
            selectedItems = m_vis.getFocusGroup(Cerebral.SELECTED);
        }

		for (int i = 0; i < orderedLabels.length; i++) {
			DecoratorItem item = orderedLabels[i];
			item.setFillColor(ColorLib.color(transparentBG));

			Object o = item.getDecoratedItem();

			if (o instanceof NodeItem) {
				VisualItem node = (VisualItem) o;
				boolean passesFilter = false;
				if (selectedItems == null) {
					passesFilter = true;
				}else if (selectedItems.containsTuple(node)) {
					passesFilter = true;
				}
			
				if (node.isHover() || node.isInGroup(Cerebral.NEIGHB_SELECT)){				    
					passesFilter = true;
				}
				

				String label = node.getString(Cerebral.LABEL);
				if (label == null || label.equalsIgnoreCase("")
						|| !node.isVisible() || !passesFilter) {
					item.setVisible(false);
				} else {

					Font baseFont = (Font) node.get(Cerebral.BASE_FONT);
					Font font = null;
					if (baseFont != null) {
						font = FontLib.getFont(baseFont.getFontName(), baseFont
								.getSize()
								* (1 / d.getScale()));
					} else {
						font = FontLib.getFont(defaultFont.getFontName(),
								defaultFont.getSize() * (1 / d.getScale()));
					}
					FontMetrics fm = Renderer.DEFAULT_GRAPHICS
							.getFontMetrics(font);
					item.setFont(font);

					PrefuseLib.setX(item, null, node.getX());
					Rectangle2D renderBounds = node.getBounds();
					PrefuseLib.setY(item, null, renderBounds.getMaxY()
							+ fm.getHeight());
					// PrefuseLib.setY(item, null, node.getY() + (node.getSize()
					// * 5) + fm.getHeight());

					tryPlaceLabel(item, false);
					item.setTextColor(cytoPrefuse.colorManager.labelColorAction
							.getColor(item));
					item
							.setFillColor(cytoPrefuse.colorManager.labelBGColorAction
									.getColor(item));
				}
			} else {
				// Aggregate label
				if (renderAggregateLabels) {
					// display function group name for groups
					AggregateItem aggItem = (AggregateItem) o;
					if (aggItem.isVisible()) {
						Integer groupID = new Integer(item
								.getInt(Cerebral.BIO_FUNCTION));
						List groupMembers = (List) functionGroups.get(groupID);
						boolean allBackground = true;	                         
						if (groupMembers != null){
						     if ( selectedItems != null){						     
						        Iterator members = groupMembers.iterator();
						        while (members.hasNext()) {
						            NodeItem member = (NodeItem) members.next();
						            if (selectedItems.containsTuple(member)){
						                allBackground =false;
						                break;						                
						            }
						            if (member.isHover() || member.isInGroup(Cerebral.NEIGHB_SELECT)){
						                allBackground = false;
						                break;
						            }
						        }
						    }else{
						        allBackground = false;
						    }
						}
						if (allBackground){
						    item.setVisible(false);
						}else if (groupMembers != null) {
							item.setVisible(true);

							double x = 0.0;
							double centerY = 0.0;
							int minY = Integer.MAX_VALUE;
							Iterator members = groupMembers.iterator();
							boolean allInvisible = true;
							while (members.hasNext()) {
								NodeItem member = (NodeItem) members.next();
								x += member.getX();
								centerY += member.getY();
								if (member.getY() < minY)
									minY = (int) member.getY();
								if (member.isVisible())
									allInvisible = false;
							}
							centerY /= groupMembers.size();

							PrefuseLib
									.setX(item, null, x / groupMembers.size());

							NodeItem aMember = (NodeItem) groupMembers
									.iterator().next();
							Rectangle2D renderBounds = aMember.getBounds();
							double y = minY - renderBounds.getHeight() / 2;

							double groupCenteringFactor = 1.0 - d.getScale();
							if (groupCenteringFactor < 0)
								groupCenteringFactor = 0;
							y += groupCenteringFactor * (centerY - y);

							PrefuseLib.setY(item, null, y);
							item.setTextColor(ColorLib.gray(0));

							Font baseFont = (Font) aMember
									.get(Cerebral.BASE_FONT);
							Font font = null;
							if (baseFont != null) {
								font = FontLib.getFont(baseFont.getFontName(),
										noScaleFontSize * aggregateScale
												* (1 / d.getScale()));
							} else {
								font = FontLib.getFont("Tahoma",
										noScaleFontSize * aggregateScale
												* (1 / d.getScale()));
							}
							item.setFont(font);
							if (allInvisible)
								item.setVisible(false);
							else
								tryPlaceLabel(item, false);
						} else {
							item.setVisible(false);
						}
					} else {
						item.setVisible(false);
					}
				} else {
					item.setVisible(false);
				}
			}
		}

	}

	private void tryPlaceLabel(DecoratorItem item,
			boolean ignoreSpaceConstraints) {
		boolean spaceEmpty = true;
		Rectangle2D renderBounds = item.getBounds();

		int xLow = (int) ((renderBounds.getMinX() / GRIDIFY) - extraSpaceX);
		if (xLow < 0)
			xLow = 0;
		if (xLow > xMax - 1)
			xLow = xMax - 1;

		int xHigh = (int) ((renderBounds.getMaxX() / GRIDIFY) + extraSpaceX);
		if (xHigh > xMax - 1)
			xHigh = xMax - 1;
		if (xHigh < 0)
			xHigh = 0;

		int yLow = (int) ((renderBounds.getMinY() / GRIDIFY) - extraSpaceY);
		if (yLow < 0)
			yLow = 0;
		if (yLow > yMax - 1)
			yLow = yMax - 1;

		int yHigh = (int) ((renderBounds.getMaxY() / GRIDIFY) + extraSpaceY);
		if (yHigh > yMax - 1)
			yHigh = yMax - 1;
		if (yHigh < 0)
			yHigh = 0;

		if (!ignoreSpaceConstraints) {
			for (int x = xLow; x <= xHigh; x++) {
				for (int y = yLow; y <= yHigh; y++) {
					if (!freeSpace[x][y]) {
						spaceEmpty = false;
						break;
					}
				}
				if (!spaceEmpty) {
					break;
				}
			}
		}
		if (spaceEmpty || ignoreSpaceConstraints) {
			if (!ignoreSpaceConstraints) {
				for (int x = xLow; x <= xHigh; x++) {
					for (int y = yLow; y <= yHigh; y++) {
						freeSpace[x][y] = false;
					}
				}
			}
			item.setVisible(true);

		} else {
			item.setVisible(false);
		}
	}

	public void prePaint(Display d, Graphics2D g) {

		// see if the sort still holds.
		boolean requiresRepaint = false;
		for (int i = 0; i < orderedLabels.length - 1; i++) {
			if (sorter.compare(orderedLabels[i], orderedLabels[i + 1]) == 1) {
				requiresRepaint = true;
				Arrays.sort(orderedLabels, sorter);
				break;
			}
		}
		VisualItem aNode = (VisualItem) cytoPrefuse.getVisualization().getVisualGroup(Cerebral.NODES).tuples().next();
		if (Math.abs(aNode.getEndX() - aNode.getX()) > 0.0001 ||
				Math.abs(aNode.getEndY() - aNode.getY()) > 0.0001){
			requiresRepaint = true;
		}
		int fontSize = (int) (noScaleFontSize * (1 / d.getScale()));
		if (fontSize != lastFontSize || requiresRepaint) {			
			lastFontSize = fontSize;
			setLabelPosition();
		}
		
	}

	public void postPaint(Display arg0, Graphics2D arg1) {
		
	}


	public void run(double frac) {
		setLabelPosition();
	}


}