package cerebral.parallelCoordinates;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import javax.swing.AbstractAction;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.InputMap;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.KeyStroke;
import javax.swing.ListModel;
import javax.swing.ListSelectionModel;

import cytoscape.Cytoscape;

public class AttributeChooser extends JPanel {

    private static final long serialVersionUID = 1L;

    public boolean cancelled = false;

    private List attributeNames;

    private JList attribNames;

    private JDialog dialog;

    public AttributeChooser() {
        attributeNames = new ArrayList();

    }

    public void showDialog() {
        removeAll();
        dialog = new JDialog(Cytoscape.getDesktop(),
                "Select expression value attributes", true);

        setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
        setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));

        String[] nodeAttribNames = Cytoscape.getNodeAttributes()
                .getAttributeNames();
        Arrays.sort(nodeAttribNames);
        String[] attribNamesWithBlank = new String[nodeAttribNames.length + 1];
        System.arraycopy(nodeAttribNames, 0, attribNamesWithBlank, 1,
                nodeAttribNames.length);
        attribNamesWithBlank[0] = new String("none");

        attribNames = new JList(attribNamesWithBlank);
        attribNames
                .setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
        attribNames.setLayoutOrientation(JList.VERTICAL);

        JScrollPane listScroller = new JScrollPane(attribNames);
        add(listScroller);

        JPanel okCancelButtonPanel = new JPanel();
        okCancelButtonPanel.setAlignmentX(Component.LEFT_ALIGNMENT);
        okCancelButtonPanel.setLayout(new BoxLayout(okCancelButtonPanel,
                BoxLayout.X_AXIS));
        okCancelButtonPanel.add(Box.createHorizontalGlue());
        JButton cancel = new JButton("Cancel");
        cancel.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent arg0) {
                cancelled = true;
                dialog.setVisible(false);
            }
        });
        okCancelButtonPanel.setAlignmentX(Component.LEFT_ALIGNMENT);
        okCancelButtonPanel.add(cancel);
        okCancelButtonPanel.add(Box.createRigidArea(new Dimension(5, 0)));

        final JButton selectAttributes = new JButton("Use selected attributes");
        selectAttributes.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent arg0) {
                Object[] selectionAsArray = attribNames.getSelectedValues();
                attributeNames.clear();
                for (int i = 0; i < selectionAsArray.length; i++) {
                    String selection = (String) selectionAsArray[i];
                    if (!selection.equals("none")) {
                        attributeNames.add(selection);
                    }
                }
                cancelled = false;

                dialog.setVisible(false);
            }

        });
        okCancelButtonPanel.add(selectAttributes);

        add(okCancelButtonPanel);

        dialog.getContentPane().add(this);
        dialog.pack();
        dialog.setLocationRelativeTo(Cytoscape.getDesktop());
        getRootPane().setDefaultButton(selectAttributes);

        attribNames.clearSelection();
        ListModel lm = attribNames.getModel();
        int[] oversizedSelectIndices = new int[lm.getSize()];
        int si = 0;
        for (int i = 0; i < lm.getSize(); i++) {
            String item = (String) lm.getElementAt(i);
            for (int j = 0; j < attributeNames.size(); j++) {
                String toSelect = (String) attributeNames.get(j);
                if (toSelect.compareToIgnoreCase(item) == 0) {
                    oversizedSelectIndices[si++] = i;
                    break;
                }
            }
        }
        int[] selectIndices = new int[si];
        System.arraycopy(oversizedSelectIndices, 0, selectIndices, 0, si);
        attribNames.setSelectedIndices(selectIndices);
        addCancelByEscapeKey();
        dialog.setVisible(true);
    }

    public List getAttributeNames() {
        return attributeNames;
    }

    public void setAttributeNames(List attributeNames) {
        this.attributeNames = attributeNames;

    }

    private void addCancelByEscapeKey() {
        String CANCEL_ACTION_KEY = "CANCEL_ACTION_KEY";
        int noModifiers = 0;
        KeyStroke escapeKey = KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE,
                noModifiers, false);
        InputMap inputMap = getRootPane().getInputMap(
                JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT);
        inputMap.put(escapeKey, CANCEL_ACTION_KEY);
          
        AbstractAction cancelAction = new AbstractAction()  {
            private static final long serialVersionUID = 1L;
            public void actionPerformed(ActionEvent e) {
                cancelled = true;
                dialog.setVisible(false);
            }
        };
        getRootPane().getActionMap().put(CANCEL_ACTION_KEY, cancelAction);
    }
}
